<?php
/**
* Подпрограммы для получения Google PageRank главной страницы сайта («морды»)
*
* @package      seoUtilitiesFiles
* @author       Andrej Panov <ab@cdef.ru>
* @copyright    2009 Andrej Panov
* @license      http://www.opensource.org/licenses/lgpl-license.php LGPL
* @version      1.20111118
* @link         http://a-panov.ru/
* @link         http://a-panov.ru/download/
* @since        2009-09-22
*/

/**
* Класс для формирования запроса на получение Google PageRank
*
* @package  seoUtilitiesClasses
*/
class cPR {

    /**
    * Получение значения Google PageRank страницы
    *
    * @param array $murl массив, возвращённый функцией cBase::get()
    * @return int значение PR
    */
    static function get($murl) {
        $host = 'http://' . $murl['host'] . '/';
        $url = 'info:' . $host;
        $ch  = self::GCH(self::strord($url));
        $url = 'info:' . urlencode($host);
        
        $request = 'http://toolbarqueries.google.com/tbr?features=Rank&sourceid=navclient-ff&client=navclient-auto-ff&ch=6' . sprintf('%u', $ch) . '&q=' . $url;
        
        $pr_str = file_get_contents($request);
        if (empty($pr_str)) return 'n/a';
        return (int) substr($pr_str, strrpos($pr_str, ":") + 1);
    }
    
    static private function nooverflow($a) {
        while ($a<-2147483648) $a+=2147483648+2147483648;
        while ($a>2147483647) $a-=2147483648+2147483648;
        return $a;
    }

    static private function zeroFill($x, $bits) {
       if ($bits==0) return $x;
       if ($bits==32) return 0;
       $y = ($x & 0x7FFFFFFF) >> $bits;
       if (0x80000000 & $x) $y |= (1<<(31-$bits));
       return $y;
    }

    static private function mix($a,$b,$c) {
        $a=(int)$a; $b=(int)$b; $c=(int)$c;
        $a -= $b; $a -= $c; $a=self::nooverflow($a); $a ^= (self::zeroFill($c,13));
        $b -= $c; $b -= $a; $b=self::nooverflow($b); $b ^= ($a<<8);
        $c -= $a; $c -= $b; $c=self::nooverflow($c); $c ^= (self::zeroFill($b,13));
        $a -= $b; $a -= $c; $a=self::nooverflow($a); $a ^= (self::zeroFill($c,12));
        $b -= $c; $b -= $a; $b=self::nooverflow($b); $b ^= ($a<<16);
        $c -= $a; $c -= $b; $c=self::nooverflow($c); $c ^= (self::zeroFill($b,5));
        $a -= $b; $a -= $c; $a=self::nooverflow($a); $a ^= (self::zeroFill($c,3));
        $b -= $c; $b -= $a; $b=self::nooverflow($b); $b ^= ($a<<10);
        $c -= $a; $c -= $b; $c=self::nooverflow($c); $c ^= (self::zeroFill($b,15));
        return array($a,$b,$c);
    }

    static private function GCH($url, $length = null, $init = 0xE6359A60) {
        if(is_null($length)) $length = sizeof($url);
        
        $a = $b = 0x9E3779B9;
        $c = $init;
        $k = 0;
        $len = $length;
        
        while($len >= 12) {
            $a += ($url[$k+0] +($url[$k+1]<<8) +($url[$k+2]<<16) +($url[$k+3]<<24));
            $b += ($url[$k+4] +($url[$k+5]<<8) +($url[$k+6]<<16) +($url[$k+7]<<24));
            $c += ($url[$k+8] +($url[$k+9]<<8) +($url[$k+10]<<16)+($url[$k+11]<<24));
            $mix = self::mix($a,$b,$c);
            $a = $mix[0]; $b = $mix[1]; $c = $mix[2];
            $k += 12;
            $len -= 12;
        }
        
        $c += $length;
        switch($len) {
            case 11: $c+=($url[$k+10]<<24);
            case 10: $c+=($url[$k+9]<<16);
            case 9 : $c+=($url[$k+8]<<8);
            case 8 : $b+=($url[$k+7]<<24);
            case 7 : $b+=($url[$k+6]<<16);
            case 6 : $b+=($url[$k+5]<<8);
            case 5 : $b+=($url[$k+4]);
            case 4 : $a+=($url[$k+3]<<24);
            case 3 : $a+=($url[$k+2]<<16);
            case 2 : $a+=($url[$k+1]<<8);
            case 1 : $a+=($url[$k]);
        }
        $mix = self::mix($a,$b,$c);
        return $mix[2];
    }

    static private function strord($string) {
        for($i = 0; $i < strlen($string); $i++) $result[$i] = ord($string[$i]);
        return $result;
    }
}
